/*
Copyright (C) 1995 Evgenii Rudnyi, http://Evgenii.Rudnyi.Ru/

Software for paper 
E. B. Rudnyi, G. F. Voronin. Thermodynamic assessment of the ternary Ba-Cu-Y system. Direct optimization of the miscibility gap. CALPHAD, 1996, v. 20, N 3, p. 297-305. 

This software is a copyrighted work licensed under the terms, described
in the file "FREE_LICENSE". 
*/

#include <math.h>
#include <iomanip.h>
#include <iostream.h>

extern double& A0BAY;
extern double& A1BAY;
extern double& A0BACUY;
extern double Tsys;
extern double A0BACU;
extern double A1BACU;
extern double A2BACU;
extern double A3BACU;
extern double A0CUY;
extern double A1CUY;
extern double A2CUY;
extern double A3CUY;
extern double R;

double DmixG(double& T, double& XCU, double& XY)
{
  double ANS2=18.*A3BACU*pow(XCU,3)*pow(XY,2)-36.*A3BACU*pow(XCU,3)*XY
     +18.*A3BACU*pow(XCU,3)+7.*A3BACU*pow(XCU,2)*pow(XY,3)
     -21.*A3BACU*pow(XCU,2)*pow(XY,2)+21.*A3BACU*pow(XCU,2)*XY
     -7.*A3BACU*pow(XCU,2)+A3BACU*XCU*pow(XY,4)-4.*A3BACU*XCU*pow(XY,3)
     +6.*A3BACU*XCU*pow(XY,2)-4.*A3BACU*XCU*XY+A3BACU*XCU+A3CUY*pow(XCU,4)*XY
     -3.*A3CUY*pow(XCU,3)*pow(XY,2)+3.*A3CUY*pow(XCU,2)*pow(XY,3)
     -A3CUY*XCU*pow(XY,4);

  double ANS1=log(-XCU-XY+1.)*R*T*XCU+log(-XCU-XY+1.)*R*T*XY-log(-XCU-XY+1.)*R*T
     -log(XCU)*R*T*XCU-log(XY)*R*T*XY+A0BACUY*pow(XCU,2)*XY
     +A0BACUY*XCU*pow(XY,2)-A0BACUY*XCU*XY+A0BACU*pow(XCU,2)+A0BACU*XCU*XY
     -A0BACU*XCU+A0BAY*XCU*XY+A0BAY*pow(XY,2)-A0BAY*XY-A0CUY*XCU*XY
     +2.*A1BACU*pow(XCU,3)+3.*A1BACU*pow(XCU,2)*XY-3.*A1BACU*pow(XCU,2)
     +A1BACU*XCU*pow(XY,2)-2.*A1BACU*XCU*XY+A1BACU*XCU+A1BAY*pow(XCU,2)*XY
     +3.*A1BAY*XCU*pow(XY,2)-2.*A1BAY*XCU*XY+2.*A1BAY*pow(XY,3)
     -3.*A1BAY*pow(XY,2)+A1BAY*XY+A1CUY*pow(XCU,2)*XY-A1CUY*XCU*pow(XY,2)
     +4.*A2BACU*pow(XCU,4)+8.*A2BACU*pow(XCU,3)*XY-8.*A2BACU*pow(XCU,3)
     +5.*A2BACU*pow(XCU,2)*pow(XY,2)-10.*A2BACU*pow(XCU,2)*XY
     +5.*A2BACU*pow(XCU,2)+A2BACU*XCU*pow(XY,3)-3.*A2BACU*XCU*pow(XY,2)
     +3.*A2BACU*XCU*XY-A2BACU*XCU-A2CUY*pow(XCU,3)*XY
     +2.*A2CUY*pow(XCU,2)*pow(XY,2)-A2CUY*XCU*pow(XY,3)
     +8.*A3BACU*pow(XCU,5)+20.*A3BACU*pow(XCU,4)*XY-20.*A3BACU*pow(XCU,4)
     +ANS2;

  return -ANS1;
}

double Dmixg1(double& T, double& XCU, double& XY)
{
  // Cu
  double ANS1=-3.*A2CUY*pow(XCU,3)*XY+6.*A2CUY*pow(XCU,2)*pow(XY,2)
     +3.*A2CUY*pow(XCU,2)*XY-3.*A2CUY*XCU*pow(XY,3)-4.*A2CUY*XCU*pow(XY,2)
     +A2CUY*pow(XY,3)+32.*A3BACU*pow(XCU,5)+80.*A3BACU*pow(XCU,4)*XY
     -100.*A3BACU*pow(XCU,4)+72.*A3BACU*pow(XCU,3)*pow(XY,2)
     -188.*A3BACU*pow(XCU,3)*XY+116.*A3BACU*pow(XCU,3)
     +28.*A3BACU*pow(XCU,2)*pow(XY,3)-117.*A3BACU*pow(XCU,2)*pow(XY,2)
     +150.*A3BACU*pow(XCU,2)*XY-61.*A3BACU*pow(XCU,2)
     +4.*A3BACU*XCU*pow(XY,4)-26.*A3BACU*XCU*pow(XY,3)
     +54.*A3BACU*XCU*pow(XY,2)-46.*A3BACU*XCU*XY+14.*A3BACU*XCU
     -A3BACU*pow(XY,4)+4.*A3BACU*pow(XY,3)-6.*A3BACU*pow(XY,2)
     +4.*A3BACU*XY-A3BACU+4.*A3CUY*pow(XCU,4)*XY
     -12.*A3CUY*pow(XCU,3)*pow(XY,2)-4.*A3CUY*pow(XCU,3)*XY
     +12.*A3CUY*pow(XCU,2)*pow(XY,3)+9.*A3CUY*pow(XCU,2)*pow(XY,2)
     -4.*A3CUY*XCU*pow(XY,4)-6.*A3CUY*XCU*pow(XY,3)+A3CUY*pow(XY,4);

  return log(XCU)*R*T+2.*A0BACUY*pow(XCU,2)*XY+2.*A0BACUY*XCU*pow(XY,2)
    -3.*A0BACUY*XCU*XY-A0BACUY*pow(XY,2)+A0BACUY*XY+A0BACU*pow(XCU,2)
    +A0BACU*XCU*XY-2.*A0BACU*XCU-A0BACU*XY+A0BACU+A0BAY*XCU*XY
    +A0BAY*pow(XY,2)-A0BAY*XY-A0CUY*XCU*XY+A0CUY*XY+4.*A1BACU*pow(XCU,3)
    +6.*A1BACU*pow(XCU,2)*XY-9.*A1BACU*pow(XCU,2)+2.*A1BACU*XCU*pow(XY,2)
    -8.*A1BACU*XCU*XY+6.*A1BACU*XCU-A1BACU*pow(XY,2)+2.*A1BACU*XY-A1BACU
    +2.*A1BAY*pow(XCU,2)*XY+6.*A1BAY*XCU*pow(XY,2)-4.*A1BAY*XCU*XY
    +4.*A1BAY*pow(XY,3)-6.*A1BAY*pow(XY,2)+2.*A1BAY*XY
    +2.*A1CUY*pow(XCU,2)*XY-2.*A1CUY*XCU*pow(XY,2)-2.*A1CUY*XCU*XY
    +A1CUY*pow(XY,2)+12.*A2BACU*pow(XCU,4)+24.*A2BACU*pow(XCU,3)*XY
    -32.*A2BACU*pow(XCU,3)+15.*A2BACU*pow(XCU,2)*pow(XY,2)
    -44.*A2BACU*pow(XCU,2)*XY+29.*A2BACU*pow(XCU,2)
    +3.*A2BACU*XCU*pow(XY,3)-16.*A2BACU*XCU*pow(XY,2)+23.*A2BACU*XCU*XY
    -10.*A2BACU*XCU-A2BACU*pow(XY,3)+3.*A2BACU*pow(XY,2)
    -3.*A2BACU*XY+A2BACU+ANS1;
}

double Dmixg2(double& T, double& XCU, double& XY)
{
  // Y
  double ANS1=-4.*A2CUY*pow(XCU,2)*XY-3.*A2CUY*XCU*pow(XY,3)+3.*A2CUY*XCU*pow(XY,2)
     +32.*A3BACU*pow(XCU,5)+80.*A3BACU*pow(XCU,4)*XY-80.*A3BACU*pow(XCU,4)
     +72.*A3BACU*pow(XCU,3)*pow(XY,2)-144.*A3BACU*pow(XCU,3)*XY
     +72.*A3BACU*pow(XCU,3)+28.*A3BACU*pow(XCU,2)*pow(XY,3)
     -84.*A3BACU*pow(XCU,2)*pow(XY,2)+84.*A3BACU*pow(XCU,2)*XY
     -28.*A3BACU*pow(XCU,2)+4.*A3BACU*XCU*pow(XY,4)
     -16.*A3BACU*XCU*pow(XY,3)+24.*A3BACU*XCU*pow(XY,2)
     -16.*A3BACU*XCU*XY+4.*A3BACU*XCU+4.*A3CUY*pow(XCU,4)*XY-A3CUY*pow(XCU,4)
     -12.*A3CUY*pow(XCU,3)*pow(XY,2)+6.*A3CUY*pow(XCU,3)*XY
     +12.*A3CUY*pow(XCU,2)*pow(XY,3)-9.*A3CUY*pow(XCU,2)*pow(XY,2)
     -4.*A3CUY*XCU*pow(XY,4)+4.*A3CUY*XCU*pow(XY,3);

  return log(XY)*R*T+2.*A0BACUY*pow(XCU,2)*XY-A0BACUY*pow(XCU,2)
    +2.*A0BACUY*XCU*pow(XY,2)-3.*A0BACUY*XCU*XY+A0BACUY*XCU
    +A0BACU*pow(XCU,2)+A0BACU*XCU*XY-A0BACU*XCU+A0BAY*XCU*XY-A0BAY*XCU
    +A0BAY*pow(XY,2)-2.*A0BAY*XY+A0BAY-A0CUY*XCU*XY+A0CUY*XCU
    +4.*A1BACU*pow(XCU,3)+6.*A1BACU*pow(XCU,2)*XY-6.*A1BACU*pow(XCU,2)
    +2.*A1BACU*XCU*pow(XY,2)-4.*A1BACU*XCU*XY+2.*A1BACU*XCU
    +2.*A1BAY*pow(XCU,2)*XY-A1BAY*pow(XCU,2)+6.*A1BAY*XCU*pow(XY,2)
    -8.*A1BAY*XCU*XY+2.*A1BAY*XCU
    +4.*A1BAY*pow(XY,3)-9.*A1BAY*pow(XY,2)+6.*A1BAY*XY-A1BAY
    +2.*A1CUY*pow(XCU,2)*XY-A1CUY*pow(XCU,2)-2.*A1CUY*XCU*pow(XY,2)
    +2.*A1CUY*XCU*XY+12.*A2BACU*pow(XCU,4)+24.*A2BACU*pow(XCU,3)*XY
    -24.*A2BACU*pow(XCU,3)+15.*A2BACU*pow(XCU,2)*pow(XY,2)
    -30.*A2BACU*pow(XCU,2)*XY+15.*A2BACU*pow(XCU,2)
    +3.*A2BACU*XCU*pow(XY,3)-9.*A2BACU*XCU*pow(XY,2)+9.*A2BACU*XCU*XY
    -3.*A2BACU*XCU-3.*A2CUY*pow(XCU,3)*XY+A2CUY*pow(XCU,3)
    +6.*A2CUY*pow(XCU,2)*pow(XY,2)+ANS1;
}

double Dmixg3(double& T, double& XCU, double& XY)
{
  // Ba
  double ANS1=42.*A3BACU*pow(XCU,2)*XY-7.*A3BACU*pow(XCU,2)
     +4.*A3BACU*XCU*pow(XY,4)-12.*A3BACU*XCU*pow(XY,3)
     +12.*A3BACU*XCU*pow(XY,2)-4.*A3BACU*XCU*XY+4.*A3CUY*pow(XCU,4)*XY
     -12.*A3CUY*pow(XCU,3)*pow(XY,2)+12.*A3CUY*pow(XCU,2)*pow(XY,3)
     -4.*A3CUY*XCU*pow(XY,4);

  return log(-XCU-XY+1.)*R*T+2.*A0BACUY*pow(XCU,2)*XY
    +2.*A0BACUY*XCU*pow(XY,2)-A0BACUY*XCU*XY+A0BACU*pow(XCU,2)
    +A0BACU*XCU*XY+A0BAY*XCU*XY+A0BAY*pow(XY,2)-A0CUY*XCU*XY
    +4.*A1BACU*pow(XCU,3)+6.*A1BACU*pow(XCU,2)*XY-3.*A1BACU*pow(XCU,2)
    +2.*A1BACU*XCU*pow(XY,2)-2.*A1BACU*XCU*XY+2.*A1BAY*pow(XCU,2)*XY
    +6.*A1BAY*XCU*pow(XY,2)-2.*A1BAY*XCU*XY+4.*A1BAY*pow(XY,3)
    -3.*A1BAY*pow(XY,2)+2.*A1CUY*pow(XCU,2)*XY-2.*A1CUY*XCU*pow(XY,2)
    +12.*A2BACU*pow(XCU,4)+24.*A2BACU*pow(XCU,3)*XY
    -16.*A2BACU*pow(XCU,3)+15.*A2BACU*pow(XCU,2)*pow(XY,2)
    -20.*A2BACU*pow(XCU,2)*XY+5.*A2BACU*pow(XCU,2)
    +3.*A2BACU*XCU*pow(XY,3)-6.*A2BACU*XCU*pow(XY,2)+3.*A2BACU*XCU*XY
    -3.*A2CUY*pow(XCU,3)*XY+6.*A2CUY*pow(XCU,2)*pow(XY,2)
    -3.*A2CUY*XCU*pow(XY,3)+32.*A3BACU*pow(XCU,5)+80.*A3BACU*pow(XCU,4)*XY
    -60.*A3BACU*pow(XCU,4)+72.*A3BACU*pow(XCU,3)*pow(XY,2)
    -108.*A3BACU*pow(XCU,3)*XY+36.*A3BACU*pow(XCU,3)
    +28.*A3BACU*pow(XCU,2)*pow(XY,3)-63.*A3BACU*pow(XCU,2)*pow(XY,2)+ANS1;
}

/*
double tmp1;
double tmp2;
double tmp3;
double& A0BAY = tmp1;
double& A1BAY = tmp2;
double& A0BACUY = tmp3;
double Tsys;
double A0BACU;
double A1BACU;
double A2BACU;
double A3BACU;
double A0CUY;
double A1CUY;
double A2CUY;
double A3CUY;
double R = 8.31441;

int main()
{
  double i;
  A0BAY = 100000.;
  A1BAY = 1.;
  A0BACUY = 1.;
  A0BACU = 100000.;
  A1BACU = 100.;
  A2BACU = 1.;
  A3BACU = 0.001;
  A0CUY = 100000.;
  A1CUY = 100.;
  A2CUY = 1.;
  A3CUY = 0.001;
  double T = Tsys;
  double xCu;
  double xY;
  double xBa;
  cout << "DmixG    xBa*muBa+xCu*muCu+xY*muY" << endl;
  for (xCu = 0.1; xCu <= 0.9; xCu += 0.1)
  {
    for (xY = 0.1; xY <= 1. - xCu - 0.1; xY += 0.1)
      cout << setw(19) << xCu << setw(19) << xY
           << setw(19)
           << DmixG(T, xCu, xY)
           << setw(19)
           << (1.-xCu-xY)*Dmixg3(T, xCu, xY) +
              xCu*Dmixg1(T, xCu, xY) + xY*Dmixg2(T, xCu, xY)
           << endl;
  }
  return 0;
}
*/

/* from main
  cout << "xY      xBa       DmixG       muBa      muY       muCu" << endl;
  for (xBa = 0.01; xBa < 1.0; xBa += 0.02)
  {
    for (xY = 0.01; xY < 1.0 ; xY += 0.02)
    {
      xCu = 1. - xY - xBa;
      cout << setw(14) << xY
           << setw(14) << xBa;
      if (xCu > 0)
        cout << setw(14) << DmixG(T, xCu, xY)
             << setw(14) << Dmixg3(T, xCu, xY)
             << setw(14) << Dmixg2(T, xCu, xY)
             << setw(14) << Dmixg1(T, xCu, xY)
             << endl;
      else
        cout << setw(14) << 0.
             << setw(14) << 0.
             << setw(14) << 0.
             << setw(14) << 0.
             << endl;
    }
  }
*/

