/*
Copyright (C) 1995 Evgenii Rudnyi, http://Evgenii.Rudnyi.Ru/

Software for paper 
E. B. Rudnyi, G. F. Voronin. Thermodynamic assessment of the ternary Ba-Cu-Y system. Direct optimization of the miscibility gap. CALPHAD, 1996, v. 20, N 3, p. 297-305. 

This software is a copyrighted work licensed under the terms, described
in the file "FREE_LICENSE". 
*/

#include <math.h>

extern double& A0BAY;
extern double& A1BAY;
extern double& A0BACUY;
extern double Tsys;
extern double A0BACU;
extern double A1BACU;
extern double A2BACU;
extern double A3BACU;
extern double A0CUY;
extern double A1CUY;
extern double A2CUY;
extern double A3CUY;
extern double R;

double dGdxCu(double& T, double& XCU, double& XY)
{
  double ANS2=-9.*A3CUY*pow(XCU,2)*pow(XY,2)+6.*A3CUY*XCU*pow(XY,3)
    -A3CUY*pow(XY,4);

  double ANS1=log(-XCU-XY+1.)*R*T-log(XCU)*R*T+2.*A0BACUY*XCU*XY
    +A0BACUY*pow(XY,2)-A0BACUY*XY+2.*A0BACU*XCU+A0BACU*XY-A0BACU+A0BAY*XY
    -A0CUY*XY+6.*A1BACU*pow(XCU,2)+6.*A1BACU*XCU*XY-6.*A1BACU*XCU
    +A1BACU*pow(XY,2)-2.*A1BACU*XY+A1BACU+2.*A1BAY*XCU*XY
    +3.*A1BAY*pow(XY,2)-2.*A1BAY*XY+2.*A1CUY*XCU*XY-A1CUY*pow(XY,2)
    +16.*A2BACU*pow(XCU,3)+24.*A2BACU*pow(XCU,2)*XY-24.*A2BACU*pow(XCU,2)
    +10.*A2BACU*XCU*pow(XY,2)-20.*A2BACU*XCU*XY+10.*A2BACU*XCU+A2BACU*pow(XY,3)
    -3.*A2BACU*pow(XY,2)+3.*A2BACU*XY-A2BACU-3.*A2CUY*pow(XCU,2)*XY
    +4.*A2CUY*XCU*pow(XY,2)-A2CUY*pow(XY,3)+40.*A3BACU*pow(XCU,4)
    +80.*A3BACU*pow(XCU,3)*XY-80.*A3BACU*pow(XCU,3)
    +54.*A3BACU*pow(XCU,2)*pow(XY,2)-108.*A3BACU*pow(XCU,2)*XY
    +54.*A3BACU*pow(XCU,2)+14.*A3BACU*XCU*pow(XY,3)
    -42.*A3BACU*XCU*pow(XY,2)+42.*A3BACU*XCU*XY-14.*A3BACU*XCU
    +A3BACU*pow(XY,4)-4.*A3BACU*pow(XY,3)+6.*A3BACU*pow(XY,2)
    -4.*A3BACU*XY+A3BACU+4.*A3CUY*pow(XCU,3)*XY+ANS2;

  return -ANS1;
}

double dGdxY(double& T, double& XCU, double& XY)
{
  return -(log(-XCU-XY+1.)*R*T-log(XY)*R*T+A0BACUY*pow(XCU,2)
    +2.*A0BACUY*XCU*XY-A0BACUY*XCU+A0BACU*XCU+A0BAY*XCU+2.*A0BAY*XY
    -A0BAY-A0CUY*XCU+3.*A1BACU*pow(XCU,2)+2.*A1BACU*XCU*XY
    -2.*A1BACU*XCU+A1BAY*pow(XCU,2)+6.*A1BAY*XCU*XY-2.*A1BAY*XCU
    +6.*A1BAY*pow(XY,2)-6.*A1BAY*XY+A1BAY+A1CUY*pow(XCU,2)
    -2.*A1CUY*XCU*XY+8.*A2BACU*pow(XCU,3)+10.*A2BACU*pow(XCU,2)*XY
    -10.*A2BACU*pow(XCU,2)+3.*A2BACU*XCU*pow(XY,2)-6.*A2BACU*XCU*XY
    +3.*A2BACU*XCU-A2CUY*pow(XCU,3)
    +4.*A2CUY*pow(XCU,2)*XY-3.*A2CUY*XCU*pow(XY,2)
    +20.*A3BACU*pow(XCU,4)+36.*A3BACU*pow(XCU,3)*XY
    -36.*A3BACU*pow(XCU,3)+21.*A3BACU*pow(XCU,2)*pow(XY,2)
    -42.*A3BACU*pow(XCU,2)*XY+21.*A3BACU*pow(XCU,2)
    +4.*A3BACU*XCU*pow(XY,3)-12.*A3BACU*XCU*pow(XY,2)
    +12.*A3BACU*XCU*XY-4.*A3BACU*XCU+A3CUY*pow(XCU,4)
    -6.*A3CUY*pow(XCU,3)*XY+9.*A3CUY*pow(XCU,2)*pow(XY,2)
    -4.*A3CUY*XCU*pow(XY,3));
}

double d2GdxCu2(double& T, double& XCU, double& XY)
{
  double ANS3=-366.*A3BACU*pow(XCU,2)*pow(XY,2)+366.*A3BACU*pow(XCU,2)*XY
    -122.*A3BACU*pow(XCU,2)+14.*A3BACU*XCU*pow(XY,4)
    -56.*A3BACU*XCU*pow(XY,3)+84.*A3BACU*XCU*pow(XY,2)
    -56.*A3BACU*XCU*XY+14.*A3BACU*XCU+12.*A3CUY*pow(XCU,4)*XY
    -6.*A3CUY*pow(XCU,3)*pow(XY,2)-12.*A3CUY*pow(XCU,3)*XY
    -12.*A3CUY*pow(XCU,2)*pow(XY,3)+18.*A3CUY*pow(XCU,2)*pow(XY,2)
    +6.*A3CUY*XCU*pow(XY,4)-6.*A3CUY*XCU*pow(XY,3)-R*T*XY+R*T;

  double ANS2=2.*A0BACUY*pow(XCU,2)*XY+2.*A0BACUY*XCU*pow(XY,2)
    -2.*A0BACUY*XCU*XY+2.*A0BACU*pow(XCU,2)+2.*A0BACU*XCU*XY
    -2.*A0BACU*XCU+12.*A1BACU*pow(XCU,3)+18.*A1BACU*pow(XCU,2)*XY
    -18.*A1BACU*pow(XCU,2)+6.*A1BACU*XCU*pow(XY,2)-12.*A1BACU*XCU*XY
    +6.*A1BACU*XCU+2.*A1BAY*pow(XCU,2)*XY+2.*A1BAY*XCU*pow(XY,2)
    -2.*A1BAY*XCU*XY+2.*A1CUY*pow(XCU,2)*XY+2.*A1CUY*XCU*pow(XY,2)
    -2.*A1CUY*XCU*XY+48.*A2BACU*pow(XCU,4)+96.*A2BACU*pow(XCU,3)*XY
    -96.*A2BACU*pow(XCU,3)+58.*A2BACU*pow(XCU,2)*pow(XY,2)
    -116.*A2BACU*pow(XCU,2)*XY+58.*A2BACU*pow(XCU,2)
    +10.*A2BACU*XCU*pow(XY,3)-30.*A2BACU*XCU*pow(XY,2)
    +30.*A2BACU*XCU*XY-10.*A2BACU*XCU-6.*A2CUY*pow(XCU,3)*XY
    -2.*A2CUY*pow(XCU,2)*pow(XY,2)+6.*A2CUY*pow(XCU,2)*XY
    +4.*A2CUY*XCU*pow(XY,3)-4.*A2CUY*XCU*pow(XY,2)
    +160.*A3BACU*pow(XCU,5)+400.*A3BACU*pow(XCU,4)*XY
    -400.*A3BACU*pow(XCU,4)+348.*A3BACU*pow(XCU,3)*pow(XY,2)
    -696.*A3BACU*pow(XCU,3)*XY+348.*A3BACU*pow(XCU,3)
    +122.*A3BACU*pow(XCU,2)*pow(XY,3)+ANS3;

  double ANS1 = ANS2/(XCU*(XCU+XY-1.));

  return -ANS1;
}

double d2GdxY2(double& T, double& XCU, double& XY)
{
  double ANS3=12.*A3CUY*pow(XCU,3)*pow(XY,2)+6.*A3CUY*pow(XCU,3)*XY
    +6.*A3CUY*pow(XCU,2)*pow(XY,3)-18.*A3CUY*pow(XCU,2)*pow(XY,2)
    -12.*A3CUY*XCU*pow(XY,4)+12.*A3CUY*XCU*pow(XY,3)-R*T*XCU+R*T;

  double ANS2=2.*A0BACUY*pow(XCU,2)*XY+2.*A0BACUY*XCU*pow(XY,2)
    -2.*A0BACUY*XCU*XY+2.*A0BAY*XCU*XY+2.*A0BAY*pow(XY,2)-2.*A0BAY*XY
    +2.*A1BACU*pow(XCU,2)*XY+2.*A1BACU*XCU*pow(XY,2)-2.*A1BACU*XCU*XY
    +6.*A1BAY*pow(XCU,2)*XY+18.*A1BAY*XCU*pow(XY,2)-12.*A1BAY*XCU*XY
    +12.*A1BAY*pow(XY,3)-18.*A1BAY*pow(XY,2)+6.*A1BAY*XY
    -2.*A1CUY*pow(XCU,2)*XY-2.*A1CUY*XCU*pow(XY,2)+2.*A1CUY*XCU*XY
    +10.*A2BACU*pow(XCU,3)*XY+16.*A2BACU*pow(XCU,2)*pow(XY,2)
    -16.*A2BACU*pow(XCU,2)*XY+6.*A2BACU*XCU*pow(XY,3)
    -12.*A2BACU*XCU*pow(XY,2)+6.*A2BACU*XCU*XY+4.*A2CUY*pow(XCU,3)*XY
    -2.*A2CUY*pow(XCU,2)*pow(XY,2)-4.*A2CUY*pow(XCU,2)*XY
    -6.*A2CUY*XCU*pow(XY,3)+6.*A2CUY*XCU*pow(XY,2)
    +36.*A3BACU*pow(XCU,4)*XY+78.*A3BACU*pow(XCU,3)*pow(XY,2)
    -78.*A3BACU*pow(XCU,3)*XY+54.*A3BACU*pow(XCU,2)*pow(XY,3)
    -108.*A3BACU*pow(XCU,2)*pow(XY,2)+54.*A3BACU*pow(XCU,2)*XY
    +12.*A3BACU*XCU*pow(XY,4)-36.*A3BACU*XCU*pow(XY,3)
    +36.*A3BACU*XCU*pow(XY,2)-12.*A3BACU*XCU*XY-6.*A3CUY*pow(XCU,4)*XY
    +ANS3;

  double ANS1=ANS2/(XY*(XCU+XY-1.));

  return -ANS1;
}

double d2GdxCudxY(double& T, double& XCU, double& XY)
{
  double ANS3=-300.*A3BACU*pow(XCU,2)*XY+150.*A3BACU*pow(XCU,2)
    +46.*A3BACU*XCU*pow(XY,3)-138.*A3BACU*XCU*pow(XY,2)
    +138.*A3BACU*XCU*XY-46.*A3BACU*XCU+4.*A3BACU*pow(XY,4)
    -16.*A3BACU*pow(XY,3)+24.*A3BACU*pow(XY,2)-16.*A3BACU*XY
    +4.*A3BACU+4.*A3CUY*pow(XCU,4)-14.*A3CUY*pow(XCU,3)*XY
    -4.*A3CUY*pow(XCU,3)+18.*A3CUY*pow(XCU,2)*XY+14.*A3CUY*XCU*pow(XY,3)
    -18.*A3CUY*XCU*pow(XY,2)-4.*A3CUY*pow(XY,4)+4.*A3CUY*pow(XY,3)+R*T;

  double ANS2=2.*A0BACUY*pow(XCU,2)+4.*A0BACUY*XCU*XY-3.*A0BACUY*XCU
    +2.*A0BACUY*pow(XY,2)-3.*A0BACUY*XY+A0BACUY+A0BACU*XCU+A0BACU*XY
    -A0BACU+A0BAY*XCU+A0BAY*XY-A0BAY-A0CUY*XCU-A0CUY*XY+A0CUY
    +6.*A1BACU*pow(XCU,2)+8.*A1BACU*XCU*XY-8.*A1BACU*XCU
    +2.*A1BACU*pow(XY,2)-4.*A1BACU*XY+2.*A1BACU+2.*A1BAY*pow(XCU,2)
    +8.*A1BAY*XCU*XY-4.*A1BAY*XCU+6.*A1BAY*pow(XY,2)-8.*A1BAY*XY
    +2.*A1BAY+2.*A1CUY*pow(XCU,2)-2.*A1CUY*XCU-2.*A1CUY*pow(XY,2)
    +2.*A1CUY*XY+24.*A2BACU*pow(XCU,3)+44.*A2BACU*pow(XCU,2)*XY
    -44.*A2BACU*pow(XCU,2)+23.*A2BACU*XCU*pow(XY,2)-46.*A2BACU*XCU*XY
    +23.*A2BACU*XCU+3.*A2BACU*pow(XY,3)-9.*A2BACU*pow(XY,2)
    +9.*A2BACU*XY-3.*A2BACU-3.*A2CUY*pow(XCU,3)+5.*A2CUY*pow(XCU,2)*XY
    +3.*A2CUY*pow(XCU,2)+5.*A2CUY*XCU*pow(XY,2)-8.*A2CUY*XCU*XY
    -3.*A2CUY*pow(XY,3)+3.*A2CUY*pow(XY,2)+80.*A3BACU*pow(XCU,4)
    +188.*A3BACU*pow(XCU,3)*XY-188.*A3BACU*pow(XCU,3)
    +150.*A3BACU*pow(XCU,2)*pow(XY,2)+ANS3;

  double ANS1=ANS2/(XCU+XY-1.);

  return -ANS1;
}

double d3GdxCu3(double& T, double& XCU, double& XY)
{
  double ANS3=-432.*A3BACU*pow(XCU,2)*XY+108.*A3BACU*pow(XCU,2)+24.*
      A3CUY*pow(XCU,5)*XY+30.*A3CUY*pow(XCU,4)*pow(XY,2)-48.*A3CUY*pow(XCU,4)*XY
      -12.*A3CUY*pow(XCU,3)*pow(XY,3)-12.*A3CUY*pow(XCU,3)*pow(XY,2)
      +24.*A3CUY*pow(XCU,3)*XY-18.*A3CUY*pow(XCU,2)*pow(XY,4)+36.*
      A3CUY*pow(XCU,2)*pow(XY,3)-18.*A3CUY*pow(XCU,2)*pow(XY,2)+2.*R*T*XCU
      *XY-2.*R*T*XCU+R*T*pow(XY,2)-2.*R*T*XY+R*T;

  double ANS2=12.*A1BACU*pow(XCU,4)+24.*A1BACU*pow(XCU,3)*XY-24.*
      A1BACU*pow(XCU,3)+12.*A1BACU*pow(XCU,2)*pow(XY,2)-24.*A1BACU*pow(XCU,2)*XY
      +12.*A1BACU*pow(XCU,2)+96.*A2BACU*pow(XCU,5)+240.*
      A2BACU*pow(XCU,4)*XY-240.*A2BACU*pow(XCU,4)+192.*A2BACU*pow(XCU,3)*pow(XY,2)
      -384.*A2BACU*pow(XCU,3)*XY+192.*A2BACU*pow(XCU,3)+
      48.*A2BACU*pow(XCU,2)*pow(XY,3)-144.*A2BACU*pow(XCU,2)*pow(XY,2)+144.
      *A2BACU*pow(XCU,2)*XY-48.*A2BACU*pow(XCU,2)-6.*A2CUY*pow(XCU,4)*
      XY-12.*A2CUY*pow(XCU,3)*pow(XY,2)+12.*A2CUY*pow(XCU,3)*XY-6.*
      A2CUY*pow(XCU,2)*pow(XY,3)+12.*A2CUY*pow(XCU,2)*pow(XY,2)-6.*A2CUY*
      pow(XCU,2)*XY+480.*A3BACU*pow(XCU,6)+1440.*A3BACU*pow(XCU,5)*XY-
      1440.*A3BACU*pow(XCU,5)+1548.*A3BACU*pow(XCU,4)*pow(XY,2)-3096.*
      A3BACU*pow(XCU,4)*XY+1548.*A3BACU*pow(XCU,4)+696.*A3BACU*pow(XCU,3)*pow(XY,3)-
      2088.*A3BACU*pow(XCU,3)*pow(XY,2)+2088.*A3BACU*pow(XCU,3)*XY
      -696.*A3BACU*pow(XCU,3)+108.*A3BACU*pow(XCU,2)*pow(XY,4)-
      432.*A3BACU*pow(XCU,2)*pow(XY,3)+648.*A3BACU*pow(XCU,2)*pow(XY,2)+
      ANS3;

  double ANS1=ANS2/(pow(XCU,2)*(pow(XCU,2)+2.*XCU*XY-2.*XCU+pow(XY,2)-2.*XY+1.));

  return -ANS1;
}

double d3GdxCu2dxY(double& T, double& XCU, double& XY)
{
  double ANS3=714.*A3BACU*pow(XCU,2)+300.*A3BACU*XCU*pow(XY,3)-900.*
      A3BACU*XCU*pow(XY,2)+900.*A3BACU*XCU*XY-300.*A3BACU*XCU+
      42.*A3BACU*pow(XY,4)-168.*A3BACU*pow(XY,3)+252.*A3BACU*pow(XY,2)-
      168.*A3BACU*XY+42.*A3BACU+12.*A3CUY*pow(XCU,4)-12.*A3CUY*
      pow(XCU,3)*XY-24.*A3CUY*pow(XCU,3)-42.*A3CUY*pow(XCU,2)*pow(XY,2)+
      48.*A3CUY*pow(XCU,2)*XY+12.*A3CUY*pow(XCU,2)+36.*A3CUY*XCU*pow(XY,2)
      -36.*A3CUY*XCU*XY+18.*A3CUY*pow(XY,4)-36.*A3CUY*pow(XY,3)
      +18.*A3CUY*pow(XY,2)-R*T;

  double ANS2=2.*A0BACUY*pow(XCU,2)+4.*A0BACUY*XCU*XY-4.*A0BACUY*
      XCU+2.*A0BACUY*pow(XY,2)-4.*A0BACUY*XY+2.*A0BACUY+6.*
      A1BACU*pow(XCU,2)+12.*A1BACU*XCU*XY-12.*A1BACU*XCU+6.*
      A1BACU*pow(XY,2)-12.*A1BACU*XY+6.*A1BACU+2.*A1BAY*pow(XCU,2)
      +4.*A1BAY*XCU*XY-4.*A1BAY*XCU+2.*A1BAY*pow(XY,2)-4.*
      A1BAY*XY+2.*A1BAY+2.*A1CUY*pow(XCU,2)+4.*A1CUY*XCU*XY-4.
      *A1CUY*XCU+2.*A1CUY*pow(XY,2)-4.*A1CUY*XY+2.*A1CUY+48.*
      A2BACU*pow(XCU,3)+116.*A2BACU*pow(XCU,2)*XY-116.*A2BACU*pow(XCU,2)
      +88.*A2BACU*XCU*pow(XY,2)-176.*A2BACU*XCU*XY+88.*
      A2BACU*XCU+20.*A2BACU*pow(XY,3)-60.*A2BACU*pow(XY,2)+60.*
      A2BACU*XY-20.*A2BACU-6.*A2CUY*pow(XCU,3)-4.*A2CUY*pow(XCU,2)
      *XY+12.*A2CUY*pow(XCU,2)+10.*A2CUY*XCU*pow(XY,2)-4.*A2CUY*
      XCU*XY-6.*A2CUY*XCU+8.*A2CUY*pow(XY,3)-16.*A2CUY*pow(XY,2)+
      8.*A2CUY*XY+240.*A3BACU*pow(XCU,4)+696.*A3BACU*pow(XCU,3)*XY-
      696.*A3BACU*pow(XCU,3)+714.*A3BACU*pow(XCU,2)*pow(XY,2)-1428.*
      A3BACU*pow(XCU,2)*XY+ANS3;

  double ANS1=ANS2/(pow(XCU,2)+2.*XCU*XY-2.*XCU+pow(XY,2)-2.*XY+1.);

  return -ANS1;
}

double d3GdxY3(double& T, double& XCU, double& XY)
{
  double ANS3=-24.*A3CUY*XCU*pow(XY,5)+48.*A3CUY*XCU*pow(XY,4)-24.*
      A3CUY*XCU*pow(XY,3)+R*T*pow(XCU,2)+2.*R*T*XCU*XY-2.*R*T*XCU-
      2.*R*T*XY+R*T;

  double ANS2=12.*A1BAY*pow(XCU,2)*pow(XY,2)+24.*A1BAY*XCU*pow(XY,3)-24.*
      A1BAY*XCU*pow(XY,2)+12.*A1BAY*pow(XY,4)-24.*A1BAY*pow(XY,3)+12.*
      A1BAY*pow(XY,2)+6.*A2BACU*pow(XCU,3)*pow(XY,2)+12.*A2BACU*pow(XCU,2)
      *pow(XY,3)-12.*A2BACU*pow(XCU,2)*pow(XY,2)+6.*A2BACU*XCU*pow(XY,4)-
      12.*A2BACU*XCU*pow(XY,3)+6.*A2BACU*XCU*pow(XY,2)-6.*A2CUY*pow(XCU,3)*pow(XY,2)
      -12.*A2CUY*pow(XCU,2)*pow(XY,3)+12.*A2CUY*pow(XCU,2)*pow(XY,2)
      -6.*A2CUY*XCU*pow(XY,4)+12.*A2CUY*XCU*pow(XY,3)-6.*A2CUY*
      XCU*pow(XY,2)+42.*A3BACU*pow(XCU,4)*pow(XY,2)+108.*A3BACU*pow(XCU,3)
      *pow(XY,3)-108.*A3BACU*pow(XCU,3)*pow(XY,2)+90.*A3BACU*pow(XCU,2)*pow(XY,4)
      -180.*A3BACU*pow(XCU,2)*pow(XY,3)+90.*A3BACU*pow(XCU,2)*pow(XY,2)
      +24.*A3BACU*XCU*pow(XY,5)-72.*A3BACU*XCU*pow(XY,4)+72.*
      A3BACU*XCU*pow(XY,3)-24.*A3BACU*XCU*pow(XY,2)+18.*A3CUY*pow(XCU,4)*pow(XY,2)
      +12.*A3CUY*pow(XCU,3)*pow(XY,3)-36.*A3CUY*pow(XCU,3)*pow(XY,2)
      -30.*A3CUY*pow(XCU,2)*pow(XY,4)+12.*A3CUY*pow(XCU,2)*pow(XY,3)+
      18.*A3CUY*pow(XCU,2)*pow(XY,2)+ANS3;

  double ANS1=ANS2/(pow(XY,2)*(pow(XCU,2)+2.*XCU*XY-2.*XCU+pow(XY,2)-2.*XY+1.));

  return -ANS1;
}

double d3GdxCudxY2(double& T, double& XCU, double& XY)
{
  double ANS3=288.*A3BACU*pow(XCU,2)+108.*A3BACU*XCU*pow(XY,3)-324.*
      A3BACU*XCU*pow(XY,2)+324.*A3BACU*XCU*XY-108.*A3BACU*XCU+
      12.*A3BACU*pow(XY,4)-48.*A3BACU*pow(XY,3)+72.*A3BACU*pow(XY,2)-
      48.*A3BACU*XY+12.*A3BACU-18.*A3CUY*pow(XCU,4)+36.*A3CUY*
      pow(XCU,3)+42.*A3CUY*pow(XCU,2)*pow(XY,2)-36.*A3CUY*pow(XCU,2)*XY-
      18.*A3CUY*pow(XCU,2)+12.*A3CUY*XCU*pow(XY,3)-48.*A3CUY*XCU*pow(XY,2)
      +36.*A3CUY*XCU*XY-12.*A3CUY*pow(XY,4)+24.*A3CUY*pow(XY,3)
      -12.*A3CUY*pow(XY,2)-R*T;

  double ANS2=2.*A0BACUY*pow(XCU,2)+4.*A0BACUY*XCU*XY-4.*A0BACUY*
      XCU+2.*A0BACUY*pow(XY,2)-4.*A0BACUY*XY+2.*A0BACUY+2.*
      A1BACU*pow(XCU,2)+4.*A1BACU*XCU*XY-4.*A1BACU*XCU+2.*
      A1BACU*pow(XY,2)-4.*A1BACU*XY+2.*A1BACU+6.*A1BAY*pow(XCU,2)+
      12.*A1BAY*XCU*XY-12.*A1BAY*XCU+6.*A1BAY*pow(XY,2)-12.*
      A1BAY*XY+6.*A1BAY-2.*A1CUY*pow(XCU,2)-4.*A1CUY*XCU*XY+4.
      *A1CUY*XCU-2.*A1CUY*pow(XY,2)+4.*A1CUY*XY-2.*A1CUY+20.*
      A2BACU*pow(XCU,3)+46.*A2BACU*pow(XCU,2)*XY-46.*A2BACU*pow(XCU,2)
      +32.*A2BACU*XCU*pow(XY,2)-64.*A2BACU*XCU*XY+32.*A2BACU*
      XCU+6.*A2BACU*pow(XY,3)-18.*A2BACU*pow(XY,2)+18.*A2BACU*XY-
      6.*A2BACU+8.*A2CUY*pow(XCU,3)+10.*A2CUY*pow(XCU,2)*XY-16.*
      A2CUY*pow(XCU,2)-4.*A2CUY*XCU*pow(XY,2)-4.*A2CUY*XCU*XY+8.*
      A2CUY*XCU-6.*A2CUY*pow(XY,3)+12.*A2CUY*pow(XY,2)-6.*A2CUY*XY
      +108.*A3BACU*pow(XCU,4)+300.*A3BACU*pow(XCU,3)*XY-300.*
      A3BACU*pow(XCU,3)+288.*A3BACU*pow(XCU,2)*pow(XY,2)-576.*A3BACU*
      pow(XCU,2)*XY+ANS3;

  double ANS1=ANS2/(pow(XCU,2)+2.*XCU*XY-2.*XCU+pow(XY,2)-2.*XY+1.);

  return -ANS1;
}
