/*
Evgenii Rudnyi, http://Evgenii.Rudnyi.Ru/
*/

#include <iomanip.h>
#include <math.h>
#include "imsl.h"

double imsl::zxssq_machine_eps = 16.8;

extern "C"{
void ss(double* x, int *m, int *n, double* f)
{
  f[0] = x[0]*pow(0e0,3) + x[1]*pow(0e0,2) + x[2] - 1.01;
  f[1] = x[0]*pow(1e0,3) + x[1]*pow(1e0,2) + x[2] - 3.03;
  f[2] = x[0]*pow(2e0,3) + x[1]*pow(2e0,2) + x[2] - 13.05;
  f[3] = x[0]*pow((-1e0),3) + x[1]*pow(1e0,2) + x[2] - 0.99;
  f[4] = x[0]*pow((-2e0),3) + x[1]*pow(2e0,2) + x[2] + 3.08;
}
}

int main()
{
      const int npoints = 200, nparam = 10;
      double x[nparam], parm[4], f[npoints],
         xjac[nparam][npoints], xjtj[nparam*(nparam+1)/2],
         work[5*nparam + 2*npoints + (nparam+1)*nparam/2], b[nparam];
      double ssq, eps, delta;
      int nsig, maxfn, iopt, m, n, infer, ier, i, j, ijob;

      nsig = 3;
      eps = 1e-4;
      delta = 0.;
      maxfn = 100;
      iopt = 1;
      n = 3;
      m = 5;
      x[0] = 0.;
      x[1] = 0.;
      x[2] = 0.;
			int npo = npoints;
      zxssq1_(ss,&m,&n,&nsig,&eps,&delta,&maxfn,&iopt,
                parm,x,&ssq,f,*xjac,&npo,xjtj,work,&infer,&ier,
								&imsl::zxssq_machine_eps);

      cout << "work ";
      for (i = 0; i < 5; i++)
        cout << setw(10) << work[i];
      cout << endl;
      cout << "infer = " << infer << "  ier = " << ier << endl;
      for (i = 0; i < n; i++)
        cout << setw(10) << x[i];
      cout << endl;
      cout << "ssq " << ssq << endl;
      for (i = 0; i < m; i++) {
        for (j = 0; j < n; j++)
          cout << setw(10) << xjac[j][i];
        cout << setw(10) << f[i] << endl;
      }
      ijob = 1;
      linv3p_(xjtj, b, &ijob, &n, &ier);
      cout << endl;
      cout << "linv3p - ier " << ier << endl;
      for (i = 0; i < n; i++) {
        for (j = i*(i+1)/2+1 - 1; j < (i+1)*(i+2)/2; j++)
          cout << setw(10) << xjtj[j];
        cout << endl;
      }
	return 0;
}
