/*
Copyright (C) 1996-1998 Evgenii Rudnyi, http://Evgenii.Rudnyi.Ru/

Software for paper 
E. B. Rudnyi, V. V. Kuzmenko, G. V. Voronin. Simultaneous assessment of the YBa2Cu6O6+z thermodynamics under the linear error model. J. Phys. Chem. Ref. Data, 1998, v. 27, N 5, p. 855-888.

This software is a copyrighted work licensed under the terms, described
in the file "FREE_LICENSE". 
*/

#include "cuox_pl.h"
#include <float.h>
#include <imsl.h>

istream& CuOx_plane::read(istream& in)
{
  clear();
  int i, k;
  token current(in);
  current.compare("<");
  current.compare("solution");
  current.compare("type");
  current.compare("=");
  current.compare("CuOx_plane");
  current.compare(">");
  current.compare("<");
  current.compare("v");
  current.compare("name");
  current.compare("=");
  current.compare("R");
  current.compare(">");
  in >> R_;
  current.compare("<");
  current.compare("/v");
  current.compare(">");
  current.compare("<");
  current.compare("name");
  current.compare(">");
  name() = current.get();
  current.compare("<");
  current.compare("/name");
  current.compare(">");
  current.compare("<");
  current.compare("components");
  current.compare(">");
  in >> comps_[0];
  in >> comps_[1];
  mol_formula::iterator i0 = comps_[0].second.begin();
  mol_formula::iterator i1 = comps_[1].second.begin();
  while (i1 != comps_[1].second.end())
  {
    if ((*i1).first == elem::O)
    {
      if ((*i0).first == elem::O)
      {
        if ((*i1++).second - (*i0++).second != 1)
          throw xmsg("components are incompatible with CuOx_plane");
      }
      else
      {
        if ((*i1++).second != 1)
          throw xmsg("components are incompatible with CuOx_plane");
      }
    }
    else
    {
      if (*i1++ != *i0++)
        throw xmsg("components are incompatible with CuOx_plane");
    }
  }
  current.compare("<");
  current.compare("/components");
  current.compare(">");

  current.compare("Gid");
  while (1)
  {
    current.compare("<");
    if (current.get() != "func_x")
      break;
    current.compare(">");
    if (current.get() == "+1*")
      { i = 0; }
    else if (current == "+z*")
      { i = 1; }
    else if (current.substr(0, 9) == "+z*(1-z)^")
    {
      k = atoi(current.substr(9).c_str());
      if (k == 0)
        throw xmsg("read: wrong token");
      i = k + 1;
      if (k > na())
      {
        fptr_.insert(fptr_.begin() + 2 + na(), k - na(), func_Tp_ptr());
        na() = k;
      }
    }
    else if (current == "+(c^2-x^2)*")
    {
      i = 2 + na();
      if (!nb())
      {
        fptr_.insert(fptr_.begin() + 2 + na(), func_Tp_ptr());
        nb()++;
      }
    }
    else if (current.substr(0, 17) == "+(c^2-x^2)*(1-z)^")
    {
      k = atoi(current.substr(17).c_str()) + 1;
      if (k == 0)
        throw xmsg("read: wrong token");
      i = k + 1 + na();
      if (k > nb())
      {
        fptr_.insert(fptr_.begin() + 2 + na() + nb(), k - nb(), func_Tp_ptr());
        nb() = k;
      }
    }
    else
      throw xmsg("read: wrong token");
    current.compare("<");
    current.compare("/func_x");
    current.compare(">");
    in >> fptr_[i];
  }
  if (current != "/solution")
      throw xmsg("read: wrong token");
  current.compare(">");
  set_coefs();
  return in;
}

ostream& CuOx_plane::write(ostream& out) const
{
  out << "<solution type=CuOx_plane> <v name=R> " << R_ << "</v> ";
  out << "<name> " << name() << " </name> " << endl;
  out << "<components> " << comps_[0] << ' ' << comps_[1] << " </components>" << endl;
  out << "Gid ";
  if (&fi(0) != NULL_FPTR)
    out << "<func_x> +1* </func_x> " << endl << fi(0);
  if (&fi(1) != NULL_FPTR)
    out << "<func_x> +z* </func_x> " << endl << fi(1);
  int i;
  for (i = 0; i < na(); ++i)
    if (&ai(i) != NULL_FPTR)
      out << "<func_x> +z*(1-z)^" << i + 1 << "* </func_x> " << endl << ai(i);
  if (nb() && &bi(0) != NULL_FPTR)
    out << "<func_x> +(c^2-x^2)* </func_x> " << endl << bi(0);
  for (i = 1; i < nb(); ++i)
    if (&bi(i) != NULL_FPTR)
      out << "<func_x> +(c^2-x^2)*(1-z)^" << i << "* </func_x> " << endl
          << bi(i);
  out << "</solution>" << endl;
  return out;
}

void CuOx_plane::set_coefs()
{
  if (!coefs_.empty()) coefs_.erase(coefs_.begin(), coefs_.end());
  vec_func_Tp_ptr::iterator i;
  vec_coef::iterator j;
  back_insert_iterator<vec_coef_ptr> to(coefs_);
  for (i = fptr_.begin(); i != fptr_.end(); ++i)
  {
    j = (*i)->begin();
    while (j != (*i)->end())
      *to++ = &(*j++);
  }
}

double CuOx_plane::Z(func_Tp::Z f, const double &T, const double &z,
  const double &x) const
{
  int i;
  double sum = 0.;
  double c = z/2.;
//  check_z(z, x);
  if (f == &func_Tp::G || f == &func_Tp::S)
  {
    if (z > 0.)
      sum += z*log(z);
    if (z < 1.)
      sum += (1. - z)*log(1. - z);
    if (c + x > 0.)
      sum += (c + x)*log(c + x);
    if (c - x > 0.)
      sum += (c - x)*log(c - x);
    if (1. - c + x > 0.)
      sum += (1. - c + x)*log(1. - c + x);
    if (1. - c - x > 0.)
      sum += (1. - c - x)*log(1. - c - x);
    if (f == &func_Tp::G)
      sum *= R_*T;
    else
      sum *= -R_;
  }

  sum += (fi(0).*f)(T, global::p);
  sum += z*(fi(1).*f)(T, global::p);
  if (na())
  {
    sum += z*(1. - z)*(ai(0).*f)(T, global::p);
    for (i = 1; i < na_; ++i)
      sum += z*(1. - z)*(ai(i).*f)(T, global::p)*pow(1. - z, i);
  }
  if (nb())
  {
    sum += (c*c - x*x)*(bi(0).*f)(T, global::p);
    for (i = 1; i < nb_; ++i)
      sum += (c*c - x*x)*(bi(i).*f)(T, global::p)*pow(1. - z, i);
  }
  return sum;
}

double CuOx_plane::dZdx(func_Tp::Z f, const double &T, const double &z,
    const double &x) const
{
  int i;
  double sum = 0.;
  double c = z/2.;
//  check_z(z, x);
  if (f == &func_Tp::G || f == &func_Tp::S)
  {
    // ??????????????????
    int sign = 0;
    if (z == 0.)
      sign = -1;
    if (x == c || z == 1.)
      sign = 1;
    if (sign != 0)
    {
      if (f == &func_Tp::S)
        sign = -sign;
      if (sign == 1)
        return HUGE_VAL;
      else
        return -HUGE_VAL;
    }
    if (c + x > 0.)
      sum += log(c + x);
    if (c - x > 0.)
      sum += -log(c - x);
    sum += log(1. - c + x);
    sum += -log(1. - c - x);
    if (f == &func_Tp::G)
      sum *= R_*T;
    else
      sum *= -R_;
  }

  if (nb())
  {
    sum -= 2.*x*(bi(0).*f)(T, global::p);
    for (i = 1; i < nb_; ++i)
      sum -= 2.*x*(bi(i).*f)(T, global::p)*pow(1. - z, i);
  }
  return sum;
}

double CuOx_plane::dZdz(func_Tp::Z f, const double &T, const double &z,
    const double &x) const
{
  int i;
  double c = z/2.;
  double sum = 0.;
  if (f == &func_Tp::G || f == &func_Tp::S)
  {
    int sign = 0;
    if (z == 0.)
      sign = -1;
    else if (x == c || z == 1.)
      sign = 1;
    if (sign != 0)
    {
      if (f == &func_Tp::S)
        sign = -sign;
      if (sign == 1)
        return HUGE_VAL;
      else
        return -HUGE_VAL;
    }
    sum += log(z);
    sum += -log(1. - z);
    sum += 0.5*log(c + x);
    sum += 0.5*log(c - x);
    sum += -0.5*log(1. - c + x);
    sum += -0.5*log(1. - c - x);
    if (f == &func_Tp::G)
      sum *= R_*T;
    else
      sum *= -R_;
  }
  sum += (fi(1).*f)(T, global::p);
  if (na())
    sum += (1. - 2.*z)*(ai(0).*f)(T, global::p);
  if (na() > 1)
  {
    sum += (ai(1).*f)(T, global::p)*((1. - 2*z)*(1. - z) - z*(1. - z));
    for (i = 2; i < na_; ++i)
      sum += (ai(i).*f)(T, global::p)*
        ((1. - 2*z)*pow(1. - z, i) - z*(1. - z)*i*pow(1. - z, i - 1));
  }
  if (nb())
    sum += c*(bi(0).*f)(T, global::p);
  if (nb() > 1)
  {
    sum += (bi(1).*f)(T, global::p)*(c*(1. - z) - (c*c - x*x));
    for (i = 2; i < nb_; ++i)
      sum += (bi(i).*f)(T, global::p)*
        (c*pow(1. - z, i) - (c*c - x*x)*i*pow(1. - z, i - 1));
  }
  return sum;
}

class solve_x
{
public:
  double T;
  double z;
  double sum;
  double R;
  solve_x(const double &T, const double &z, const double &sum,
          const double &R)
    : T(T), z(z), sum(sum), R(R) {}
  double operator()(const double &x)
  {
    double c = z/2.;
    double res;
    res = R*T*log((c + x)*(1. - c + x)/(c - x)/(1. - c - x)) - 2.*x*sum;
//    cout << "  solve_x" << setw(10) << x << setw(10) << res << endl;
    return res;
  }
};

static solve_x *solve_x_ptr;

extern "C" double zbrent_solve(double *x)
{
  return solve_x_ptr->operator()(*x);
}

double CuOx_plane::x_eq(const double &T, const double &z) const
{
//  check_z(z);
  int i;
  double c = z/2.;
  double eps = sqrt(DBL_EPSILON);
  double x;
  if (c < eps)
    return 0.;
  double sum = 0.;
  if (nb())
  {
    sum += bi(0).G(T);
    for (i = 1; i < nb_; ++i)
      sum += bi(i).G(T)*pow(1. - z, i);
  }
  if (T/c/(1. - c) - sum >= 0.)
    x = 0.;
  else
  {
    double a = c*eps;
    double b = c*(1.-eps);
    solve_x solx(T, z, sum, R_);
		solve_x_ptr = &solx;
    double eps = 0.;
    int nsig = 10;
    int maxfn = 100;
		int ier;
    zbrent_(zbrent_solve, &eps, &nsig, &a, &b, &maxfn, &ier);
    if (ier == 130)
		{
      if (G(T, z, 0.) < G(T, z, c))
        x = 0.;
      else
        x = c;
    }
		else
     x = b;
 }
  return x;
}

/*
  DEPEND sum, z;
  DEPEND c, z;
  OFF EXP;
  ON GCD;
  ON EZGCD;
  f := R*T*log((c + x)*(1 - c + x)/(c - x)/(1 - c - x)) - 2*x*sum;
  %  c := z/2;
  dfdx := df(f, x);
  dfdz := df(f, z);
  LET df(c, z) = 1/2;
  dfdz := dfdz;
  dxdz := -dfdz/dfdx;
  ON FORT;
  dxdz;
      ANS=(((2.*C-1.)*R*T-2.*(C+X-1.)*(C+X)*(C-X-1.)*(C-X)*
     . DF(SUM,Z))*X)/(2.*((C**2-C+X**2)*R*T+(C+X-1.)*(C+X)*
     . (C-X-1.)*(C-X)*SUM))
*/

double CuOx_plane::dxdz(const double &T, const double &z,
  const double &x) const
{
//  check_z(z, x);
  int i;
  double c = z/2.;
  double sum = 0.;
  if (z == 0.)
    return 0.;
  if (nb())
  {
    sum = bi(0).G(T);
    for (i = 1; i < nb_; ++i)
      sum += bi(i).G(T)*pow(1. - z, i);
  }
  double dsumdz = 0.;
  if (nb() > 1)
  {
    dsumdz = -bi(1).G(T);
    for (i = 2; i < nb_; ++i)
      dsumdz += -i*bi(i).G(T)*pow(1. - z, i - 1);
  }
  return
     (((2.*c-1.)*R_*T-2.*(c+x-1.)*(c+x)*(c-x-1.)*(c-x)*
     dsumdz)*x)/(2.*((pow(c, 2)-c+pow(x, 2))*R_*T+(c+x-1.)*(c+x)*
     (c-x-1.)*(c-x)*sum));
}

/*
  DEPEND sum, T;
  OFF EXP;
  ON GCD;
  ON EZGCD;
  f := R*T*log((c + x)*(1 - c + x)/(c - x)/(1 - c - x)) - 2*x*sum;
  dfdx := df(f, x);
  dfdT := df(f, T);
  dxdT := -dfdT/dfdx;
  ON FORT;
  dxdT;
      ANS=-((2.*DF(SUM,T)*X-LOG(((C+X)*(C-X-1.))/((C+X-1.)*
     . (C-X)))*R)*(C+X-1.)*(C+X)*(C-X-1.)*(C-X))/(2.*((C**2
     . -C+X**2)*R*T+(C+X-1.)*(C+X)*(C-X-1.)*(C-X)*SUM))
*/

double CuOx_plane::dxdT(const double &T, const double &z,
  const double &x) const
{
//  check_z(z, x);
  int i;
  double c = z/2.;
  if (z == 0)
    return 0;
  double sum = 0.;
  double dsumdT = 0.;
  if (nb())
  {
    sum = bi(0).G(T);
    dsumdT = -bi(0).S(T);
    for (i = 1; i < nb_; ++i)
    {
      sum += bi(i).G(T)*pow(1. - z, i);
      dsumdT -= bi(i).S(T)*pow(1. - z, i);
    }
  }
//  if (x == c/2.) -  ᤥ
  return
      -((2.*dsumdT*x-log(((c+x)*(c-x-1.))/((c+x-1.)*
      (c-x)))*R_)*(c+x-1.)*(c+x)*(c-x-1.)*(c-x))/(2.*((pow(c, 2)
      -c+pow(x, 2))*R_*T+(c+x-1.)*(c+x)*(c-x-1.)*(c-x)*sum));
}

static double ln_tmp;
static double T_tmp;
static CuOx_plane *Y123_tmp;

void xtoz(double x, double& z)
{
  z = log(x/(1. - x));
}

void ztox(double& x, double z)
{
  if (z > -600)
    x = 1./(1 + exp(-z));
  else
    x = 1e-265;
}

extern "C" {
static double pO2_solve(double *z)
{
  double z_good;
  ztox(z_good, *z);
  return ln_tmp - Y123_tmp->lnpO2(T_tmp, z_good);
}

static double pO2_solve_1(double *z)
{
  return ln_tmp - Y123_tmp->lnpO2(T_tmp, *z);
}

}

double CuOx_plane::z(const double &T, const double &lnpO2,
                     double z_init)
{
  double eps = 0., eps2 = 0., eta = 0.;
  double x, res;
  int nsig = 11, n = 1, itmax = 80, ier;
  xtoz(z_init, x);
  Y123_tmp = this;
  T_tmp = T;
  ln_tmp = lnpO2;
  zreal2_(pO2_solve, &eps, &eps2, &eta, &nsig, &n, &x, &itmax, &ier);
  if (ier)
  {
    itmax = 100;
    double a = DBL_EPSILON*10;
    if (this->lnpO2(T, a) > lnpO2)
      return 0.;
    double b = 1. - a;
    if (this->lnpO2(T, b) < lnpO2)
      return 1.;
    zbrent_(pO2_solve_1, &eps, &nsig, &a, &b, &itmax, &ier);
    if (ier == 130)
    {
      cout << "CuOx_plane::z - can not find z, ier = " << ier << endl;
      exit(1);
    }
    return b;
  }
  ztox(res, x);
  return res;
}

double CuOx_plane::T_tr(const double &z)
{
  double c = z/2.;
  double s1 = 0., s2 = 0.;
  for (int i = 0; i < nb(); ++i)
  {
    try
    {
      dynamic_cast<Cp_zero&>(bi(i));
    }
    catch (...)
    {
      cout << "wrong type of Tp-function - can not be handled by T_tr"
           << endl;
      exit(1);
    }
    if (i == 0)
    {
      s1 = bi(i).H(1000.);
      s2 = bi(i).S(1000.);
//      cout << i << " " << s1 << " " << s2 << endl;
    }
    else
    {
      s1 += bi(i).H(1000.)*pow((1. - z), i);
      s2 += bi(i).S(1000.)*pow((1. - z), i);
//      cout << i << " " << s1 << " " << s2 << endl;
    }
  }
  return c*(1. - c)*s1/(1. + c*(1. - c)*s2);
}

class bad_T
{
};

extern "C" double pO2_solve_T(double *z_)
{
	double &z = *z_;
  double z_good;
  ztox(z_good, z);
  double T = Y123_tmp->T_tr(z_good);
//  if (T < 200. || T > 6000.) throw bad_T();
  if (T < 1.) throw bad_T();
  double lnp = Y123_tmp->lnpO2(T, z_good);
  if (lnp > 1e100) lnp = 1e100;
  if (lnp < -1e100) lnp = -1e100;
  return ln_tmp - lnp;
}

double CuOx_plane::T_tr1(const double &lnpO2,
                        double T_tr_init)
{
  double eps = 0., eps2 = 0., eta = 0.;
  double x, res;
  int nsig = 11, n = 1, itmax = 200, ier;
  double z_init = z(T_tr_init, lnpO2);
  xtoz(z_init, x);
  Y123_tmp = this;
  ln_tmp = lnpO2;
  try
  {
    zreal2_(pO2_solve_T, &eps, &eps2, &eta, &nsig, &n, &x, &itmax, &ier);
  }
  catch (bad_T)
  {
    ier = 1000;
  }
  if (ier)
  {
    itmax = 200;
    xtoz(z_init, x);
    try
    {
      zreal1_(pO2_solve_T, &eps, &eps2, &eta, &nsig, &n, &x, &itmax, &ier);
    }
    catch (bad_T)
    {
      cout << "warning - Ttr1" << endl;
      return 0.;
    }
    if (ier)
    {
      cout << "CuOx_plane::T_tr1 - can not find T_tr, ier = " << ier << endl;
      return 0;
//      exit(1);
    }
  }
  ztox(res, x);
  return T_tr(res);
}

