/*
Copyright (C) 1996-1998 Evgenii Rudnyi, http://Evgenii.Rudnyi.Ru/

Software for paper 
E. B. Rudnyi, V. V. Kuzmenko, G. V. Voronin. Simultaneous assessment of the YBa2Cu6O6+z thermodynamics under the linear error model. J. Phys. Chem. Ref. Data, 1998, v. 27, N 5, p. 855-888.

This software is a copyrighted work licensed under the terms, described
in the file "FREE_LICENSE". 
*/

#include "formula.h"
#include <strstream>
#include <ctype.h>      // isspace

istream& mol_formula::read(istream& in)
{
  clear();
  if (!in) return in;
  elem t;
  int coef;
  while (true)
  {
    t.read(in);
    if (!t)
    {
      if (in.peek() == '(')
      {
        in.ignore();
        mol_formula t(in);
        if (in.peek() == ')')
        {
          in.ignore();
          if (isspace(in.peek()))
            coef = 1;
          else
          {
            char sign = in.peek();
            in >> coef;
            if (in.fail())
            {
              in.clear();
              coef = 1;
              if (sign == '+' || sign == '-')
                in.putback(sign);
            }
          }
          mol_formula::const_iterator i;
          for (i = t.begin(); i != t.end(); ++i)
            operator[]((*i).first) += (*i).second*coef;
        }
        else
        {
          clear();
          return in;
        }
        continue;  //check this out
      }
      if (in.peek() == '+')
      {
        in.ignore();
        --operator[](elem::e);
      }
      if (in.peek() == '-')
      {
        in.ignore();
        ++operator[](elem::e);
      }
      mol_formula::iterator i;
      for (i = begin(); i != end(); ++i)
        if (!(*i).second) erase(i);
      return in;
    }
    else
    {
      if (isspace(in.peek()))
        coef = 1;
      else
      {
        char sign = in.peek();
        in >> coef;
        if (in.fail())
        {
          in.clear();
          coef = 1;
          if (sign == '+' || sign == '-')
            in.putback(sign);
        }
      }
      operator[](t) += coef;
    }
  }
}

ostream& mol_formula::write(ostream& out) const
{
  mol_formula::const_iterator i;
  for (i = begin(); i != end(); ++i)
  {
    out << (*i).first;
    if ((*i).second != 1)
      out << (*i).second;
  }
  return out;
}

int mol_formula::noa(elem el) const
{
  mol_formula::const_iterator i = find(el);
  if (i == end())
    return 0;
  else
    return (*i).second;
}

double mol_formula::mass() const
{
  double mass = 0.;
  mol_formula::const_iterator i;
  for (i = begin(); i != end(); ++i)
    mass += (*i).second*(*i).first.mass();
  return mass;
}

istream& formula::read(istream& in)
{
  clear();
  if (!in) return in;
  elem t;
  int coef;
  ostrstream tmp;
  while (true)
  {
    t.read(in);
    if (!t)
    {
      if (in.peek() == '(')
      {
        in.ignore();
        formula t(in);
        if (in.peek() == ')')
        {
          in.ignore();
          if (isspace(in.peek()))
            coef = 1;
          else
          {
            char sign = in.peek();
            in >> coef;
            if (in.fail())
            {
              in.clear();
              coef = 1;
              if (sign == '+' || sign == '-')
                in.putback(sign);
            }
          }
          if (coef)
          {
            tmp << '(' << t.first << ')';
            if (coef != 1)
              tmp << coef;
            mol_formula::const_iterator i;
            for (i = t.mf().begin(); i != t.mf().end(); ++i)
              second[(*i).first] += (*i).second*coef;
          }
        }
        else
        {
          clear();
          return in;
        }
        continue;  //check this out
      }
      if (in.peek() == '+')
      {
        tmp << '+';
        in.ignore();
        --second[elem::e];
      }
      if (in.peek() == '-')
      {
        tmp << '-';
        in.ignore();
        ++second[elem::e];
      }
      tmp << '\0';
      first = tmp.str();
      delete tmp.str();
      mol_formula::iterator i;
      for (i = second.begin(); i != second.end(); ++i)
        if (!(*i).second) second.erase(i);
      return in;
    }
    else
    {
      if (isspace(in.peek()))
        coef = 1;
      else
      {
        char sign = in.peek();
        in >> coef;
        if (in.fail())
        {
          in.clear();
          coef = 1;
          if (sign == '+' || sign == '-')
            in.putback(sign);
        }
      }
      if (coef)
      {
        tmp << t;
        if (coef != 1)
          tmp << coef;
        second[t] += coef;
      }
    }
  }
}

