/*
Copyright (C) 1996-1998 Evgenii Rudnyi, http://Evgenii.Rudnyi.Ru/

Software for paper 
E. B. Rudnyi, V. V. Kuzmenko, G. V. Voronin. Simultaneous assessment of the YBa2Cu6O6+z thermodynamics under the linear error model. J. Phys. Chem. Ref. Data, 1998, v. 27, N 5, p. 855-888.

This software is a copyrighted work licensed under the terms, described
in the file "FREE_LICENSE". 
*/

#ifndef __FORMULA_H
#define __FORMULA_H

#include <iostream.h>

#include <string>

#include <map.h>

#include "elem.h"

class Int
{
public:
  int i;
  Int()
    {i = 0;}
  Int(int i_)
    {i = i_;}
  Int& operator=(int i_)
    {i = i_; return *this;}
  Int& operator+=(int i_)
    {i += i_; return *this;}
  Int& operator-=(int i_)
    {i -= i_; return *this;}
  Int& operator++()
    {i++; return *this;}
  Int operator++(int i_)
    {Int t = *this; i++; return t;}
  Int& operator--()
    {i--; return *this;}
  Int operator--(int i_)
    {Int t = *this; i--; return t;}
  operator int() const
    {return i;}
};

class mol_formula : public map <elem, Int, greater<elem> >
{
public:
  mol_formula()
    {}
  mol_formula(istream& in)
    {read(in);}
  ~mol_formula()
    {}
  istream& read(istream& in);
  ostream& write(ostream& out) const;
  operator int() const
    {return size();}
  void clear()
    {erase(begin(), end());}

  int nel() const
    {return size();}
  int noa(elem el) const;
  double mass() const;
};

inline istream& operator>>(istream &in, mol_formula &to)
{
  in >> ws;
  return to.read(in);
};

inline ostream& operator<<(ostream &out, const mol_formula &old)
{
  return old.write(out);
};

/*
inline bool operator==(const pair<const elem, Int> &x, const pair<const elem, Int> &y)
{
  return (x.first == y.first) && (x.second == y.second);
}

inline bool operator!=(const pair<const elem, Int> &x, const pair<const elem, Int> &y)
{
  return (x.first != y.first) || (x.second != y.second);
}

inline bool operator<(const pair<const elem, Int>& x, const pair<const elem, Int>& y)
{
    return x.first < y.first || (!(y.first < x.first) && x.second < y.second); 
}
*/

class formula : public pair<string, mol_formula>
{
public:
  formula()
    {}
  formula(istream& in)
    {read(in);}
  ~formula()
    {}
  istream& read(istream& in);
  ostream& write(ostream& out) const
    {out << first; return out;}
  ostream& write_mf(ostream& out) const
    {return second.write(out);}
  operator int() const
    {return second.size();}
  void clear()
    {first.clear(); second.clear();}

  string& name()
    {return first;}
  const string& name() const
    {return first;}
  mol_formula& mf()
    {return second;}
  const mol_formula& mf() const
    {return second;}
  int nel() const
    {return second.size();}
  int noa(elem el) const
    {return second.noa(el);}
  double mass() const
    {return second.mass();}
};

inline istream& operator>>(istream &in, formula &to)
{
  in >> ws;
  return to.read(in);
};

inline ostream& operator<<(ostream &out, const formula &old)
{
  return old.write(out);
};

#endif
