/*
Copyright (C) 1996-1998 Evgenii Rudnyi, http://Evgenii.Rudnyi.Ru/

Software for paper 
E. B. Rudnyi, V. V. Kuzmenko, G. V. Voronin. Simultaneous assessment of the YBa2Cu6O6+z thermodynamics under the linear error model. J. Phys. Chem. Ref. Data, 1998, v. 27, N 5, p. 855-888.

This software is a copyrighted work licensed under the terms, described
in the file "FREE_LICENSE". 
*/

#include "func_tp.h"
#include <math.h>

istream& func_Tp::read(istream& in)
{
  token current(in);
  if (!label_[0].empty())
    current.compare(label_[0]);
  for (int i = 0; i != size(); ++i)
  {
    in >> operator[](i);
    if (!label_[i + 1].empty())
      current.compare(label_[i + 1]);
  }
  current.compare("<");
  current.compare("/func_Tp");
  current.compare(">");
  return in;
}

ostream& func_Tp::write(ostream& out) const
{
  out << "<func_Tp type=" << ClassName() << "> ";
  out << label_[0] << ' ';
  for (int i = 0; i != size(); ++i)
    out << operator[](i) << label_[i + 1] << ' ';
  out << "</func_Tp>" << endl;
  return out;
}

double func_Tp::H(const double &T, const double& p) const
{
  double Gc = G(T, p);
  double stepT = global::step1*(T + global::step1);
  double T1 = T + stepT;
  stepT = T1 - T;
  double Sn = -(G(T1, p) - Gc)/stepT;
  return Gc + T*Sn;
}

double func_Tp::S(const double &T, const double& p) const
{
  double Gc = G(T, p);
  double stepT = global::step1*(T + global::step1);
  double T1 = T + stepT;
  stepT = T1 - T;
  return -(G(T1, p) - Gc)/stepT;
}

double func_Tp::Cp(const double &T, const double& p) const
{
  double stepT = global::step2*(T + global::step2);
  double T1 = T - stepT;
  double T2 = T + stepT;
  double df = (G(T1, p) - 2.*G(T, p) + G(T2, p))/stepT/stepT;
  return -T*df;
}

double func_Tp::V(const double &T, const double& p) const
{
  double Gc = G(T, p);
  double stepp = global::step1*(p + global::step1);
  double p1 = p + stepp;
  stepp = p1 - p;
  return -(G(T, p1) - Gc)/stepp;
}

func_Tp_ptr::func_Tp_ptr(int, int)
{
  types["null_func_Tp"].reset(new null_func_Tp);
  types["Cp_zero"].reset(new Cp_zero);
  types["Cp_zero_V"].reset(new Cp_zero_V);
  types["Cp_const"].reset(new Cp_const);
  types["Cp_BB2"].reset(new Cp_BB2);
  types["Cp_BB2_V"].reset(new Cp_BB2_V);
  types["Cp_BB2_V2"].reset(new Cp_BB2_V2);
  types["Cp_BB2_V3"].reset(new Cp_BB2_V3);
  types["Cp_BB2_V3_"].reset(new Cp_BB2_V3_);
  types["IVT_Tp"].reset(new IVT_Tp);
  types["Cp_BB4"].reset(new Cp_BB4);
}

istream& func_Tp_ptr::read(istream& in)
{
  token current(in);
  current.compare("<");
  current.compare("func_Tp");
  current.compare("type");
  current.compare("=");
  current.get();
  map_types::const_iterator i = types.find(current);
  if (i == types.end())
    throw xmsg("no such type for func_Tp");
  else
    reset((*i).second->create());
  current.compare(">");
  get()->read(in);
  return in;
}

